export interface TagTextContentParams {
    data: Uint8Array;
    tag: Uint8Array;
    pos: number;
}

export interface ParserState {
    content: string;
    pos: number;
}

const openAngleBracket: number = 60;
const closeAngleBracket: number = 62;

export function getTagTextContent (params: TagTextContentParams): ParserState {
    const {pos, tag, data} = params;
    const state: ParserState = {
        content: '',
        pos
    };
    const tagLength: number = tag.length;
    let i: number = 0;

    while (i < tagLength && tag[i] === data[pos + i]) {
        i++;
    }

    if (i === tagLength) {
        const {length} = data;
        let foundCloseAngleBracket: boolean = false;

        i = pos + i;
        let symbolCode: number = data[i];

        while (i < length && symbolCode !== openAngleBracket) {
            if (foundCloseAngleBracket) {
                state.content += String.fromCharCode(symbolCode);
            } else if (symbolCode === closeAngleBracket) {
                foundCloseAngleBracket = true;
            }

            i++;
            symbolCode = data[i];
        }

        state.pos = i;
    }

    return state;
}