import {IncomingData, IncomingFile, IncomingFileContent, FileBuffer, FiwoFile} from './typings';

export type ReaderConstructor = any;
const readers: ReaderConstructor[] = [];

export function registerReader (RConstructor: ReaderConstructor) {
    readers.push(RConstructor);
}

export function read (file: IncomingFile): Promise<FiwoFile|void> {
    return new Promise<FiwoFile|void>((resolve, reject) => {
        if (typeof Buffer !== 'undefined' && file instanceof Buffer) {
            readFileBuffer(<FileBuffer>file);
        } else if (
            (typeof File !== 'undefined' && file instanceof File) ||
            (typeof Blob !== 'undefined' && file instanceof Blob)
        ) {
            const reader: FileReader = new FileReader();

            reader.onload = () => readFileBuffer(<FileBuffer>reader.result);
            reader.onerror = reject;
            reader.readAsArrayBuffer(file);
        } else {
            reject(new Error('The incoming file should be Buffer|Blob|File type'));
        }

        function readFileBuffer (fileBuffer: FileBuffer) {
            let reader: Reader;
            const fileContent: IncomingFileContent = <IncomingFileContent>(
                fileBuffer instanceof ArrayBuffer ? new Uint8Array(fileBuffer) : fileBuffer
            );
            const incomingData: IncomingData = {
                fileBuffer,
                fileContent
            };

            readers.some((RConstructor: ReaderConstructor) => {
                if (RConstructor.testFile(incomingData)) {
                    reader = new RConstructor(fileBuffer);
                    return true;
                }
            });

            if (!reader) {
                return reject(new Error('Fiwo reader not found'));
            }

            resolve(reader.read(incomingData));
        }
    });
}

export class Reader {
    read (data: IncomingData): Promise<FiwoFile|void> {
        return Promise.reject(new Error('Invalid file'));
    }

    static testFile (data: IncomingData): Promise<boolean> {
        return Promise.resolve(false);
    }
}